"""
Extracts data from a GSSHA model solution and formats it to be read in by TSPROC as part of a PEST calibration. The
user must specify the project name and the physics to be extracted. It is assumed that all project solution files
have the same filename as the project name with the standard file extensions. For gridded parameters, the user
must give the x and y indices of each cell for which a time series should be extracted. for multi-column data, such as 
hydrographs, the column number must be given. It is highly recommended that the user test the script after entering the
target information to ensure that it is properly configured.

If a simulation doesn't have full time series matching the measured *.prn file for the timeseries, zeros are entered
for the complete series. This is intended to show that the solution parameter set is unstable and should be be considered
a viable set. A prn file must therefore exist for each timeseries prior to the execution of this script in order
to establish the duration of the timeseries. The simplest method to create an initial series is to copy the measured 
timeseries with any required adjustments.

To use this script, the user should modify the entries in the user inputs section. Below are the steps for using this
script:

1) Specify the project name
2) Flag the physics timeseries to be extracted from the solution
3) Enter the target information for each active time series
4) TEST THE SCRIPT WITH A SOLVED MODEL TO MAKE SURE YOUR SETUP IS CORRECT.
   
Author: Drew Allan Loney, PhD PE
Email: drew.a.loney@usace.army.mil
Version: 1.0

It is requested that any errors found within the script be conveyed to the script author. These will be corrected
in a timely manner, with an updated script posted for the community's use.


This script is released under the MIT license:   
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
documentation files (the "Software"), to deal in the Software without restriction, including without limitation the 
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to 
permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""


### Imports ###
import os, datetime, calendar
import numpy as np

##### USER SPECIFIED INPUTS #####
s_project_name = '2017_08_16_270m'              # Project name.  All GSSHA files are assumed to use this stem name.

# Extraction toggles
b_wse_lake = True                      # Toggle to activate wse lake extraction (.lel)
b_hydrograph = True                    # Toggle to activate hydrograph extraction (.ohl)
b_soil_moisture = False                # Toggle to activate soil/surf moisture extraction (.sur)

b_snow_depth = False                    # Toggle to activate snow depth extraction (.sdd)
b_snow_water_equivalent = False         # Toggle to activate snow water equivalent extraction (.swd)

## Time values ##
s_time_start_gridded = '2004-12-01 00:00:00'    # Start time for gridded outputs

## WSE targets ##
ia_wse_column_indices = [5]             # GSSHA column index for lake (zero indexing)
sa_outputs_wse = ['coy']                # Site names for WSE measurements

## Hydrograph targets ##
ia_hydrograph_indices = [1, 2, 3]       # Entry location in the ihl file to be processed
sa_outputs_hydrograph = ['11462500', '11461500', '11461000']  # Site name for the hydrograph measurements

## Soil Moisture targets ##
ia_soilm_cell_i = [82]                  # GSSHA cell index for i location
ia_soilm_cell_j = [67]                  # GSSHA cell index for j location
sa_outputs_soilm = ['site_1']           # Site names for WSE measurements

## Snow Depth targets ##
ia_snowdepth_cell_i = [1]               # GSSHA cell index for i location
ia_snowdepth_cell_j = [1]               # GSSHA cell index for j location
sa_outputs_snowdepth = ['site_1']       # Site names for snow depth measurements

## Snow Water Equivalent targets ##
ia_swe_cell_i = [1]                     # GSSHA cell index for i location
ia_swe_cell_j = [1]                     # GSSHA cell index for j location
sa_outputs_swe = ['site_1']             # Site names for snow water equivalent measurements





##### BEGIN MAIN BRIDGING CODE. MODIFICATION OF THIS IS GREATLY DISCOURAGED UNDER PENALTY OF WRATH #####
### Define processing flags ###
b_outputReadFailure = False

### Convert the times to datetime objects ###
o_time_start_gridded = datetime.datetime.strptime(s_time_start_gridded, "%Y-%m-%d %H:%M:%S")

### Define function to read gridded outputs ###
def read_gridded_data(s_filename_target, s_filename_elevation):
    """
    
    Parameters
    ----------
    s_filename_target: Filename of target gridded produced to be read.  Includes the extension.
    s_filename_elevation: Filename of the elevation file. This is necessary to estabilish the domain.

    Returns
    -------
    dm_gridded_data: Numpy matrix containing the gridded data at each timestep
    da_gridded_timesteps: Numpy array containing the timestep values indexed to zero at the start

    """
    
    # Read in the elevation file to get the shape of the domain
    dm_elevations = np.loadtxt(s_filename_elevation, dtype=float, skiprows=6)

    # Define holders for the variables
    da_gridded_timesteps = []
    dm_gridded_data = []
    da_gridded_data_temp = []

    # Begin reading the file
    o_gridded_file = open(s_filename_target, buffering=1024)
    s_line = o_gridded_file.readline()

    while s_line:
        # Strip the new line character from the line
        s_line = s_line.strip('\r\n')

        # Split on a space
        sa_line_split = s_line.split(' ')

        # Handle each line
        if sa_line_split[0] == 'ENDDS':
            # End of file.  Append the last timestep
            # Promote to temp data to a numpy array
            da_gridded_data_temp = np.array(da_gridded_data_temp)

            # Slice out the gridded values and avoid the mask
            da_gridded_data_temp = da_gridded_data_temp[int(len(da_gridded_data_temp) / 2):]

            # Reshape and store the array
            dm_timestep_data_reshaped = np.reshape(np.array(da_gridded_data_temp), np.shape(dm_elevations), order='C')
            dm_gridded_data.append(dm_timestep_data_reshaped)
            da_gridded_data_temp = []

        elif sa_line_split[0] == 'TS':
            # This is a new timestep. Record the time and format the previous timestep values
            if len(da_gridded_timesteps) == 0:
                # First timestep.  Do not append
                # Translate all values into seconds
                if sa_line_split[1] == '0':
                    da_gridded_timesteps.append(float(sa_line_split[2]))
                elif sa_line_split[1] == '1':
                    da_gridded_timesteps.append(float(sa_line_split[2]) * 60)
                elif sa_line_split[2] == '2':
                    da_gridded_timesteps.append(float(sa_line_split[2]) * 60 * 60)
                elif sa_line_split[2] == '3':
                    da_gridded_timesteps.append(float(sa_line_split[2]) * 60 * 60 * 24)

            else:
                # Not the first timetep.  Append and clear
                # Promote to temp data to a numpy array
                da_gridded_data_temp = np.array(da_gridded_data_temp)

                # Slice out the gridded values and avoid the mask
                da_gridded_data_temp = da_gridded_data_temp[int(len(da_gridded_data_temp) / 2):]

                # Reshape and store the array
                dm_timestep_data_reshaped = np.reshape(np.array(da_gridded_data_temp), np.shape(dm_elevations), order='C')
                dm_gridded_data.append(dm_timestep_data_reshaped)
                da_gridded_data_temp = []

                # Add the timestep value to the timestep array
                # Translate all values into seconds
                if sa_line_split[1] == '0':
                    da_gridded_timesteps.append(float(sa_line_split[2]))
                elif sa_line_split[1] == '1':
                    da_gridded_timesteps.append(float(sa_line_split[2]) * 60)
                elif sa_line_split[2] == '2':
                    da_gridded_timesteps.append(float(sa_line_split[2]) * 60 * 60)
                elif sa_line_split[2] == '3':
                    da_gridded_timesteps.append(float(sa_line_split[2]) * 60 * 60 * 24)

        else:
            # Try to parse to a double
            try:
                # Cast to float
                d_value_cell = float(sa_line_split[0])

                # Cast was successful.  Put into the timestep array
                da_gridded_data_temp.append(d_value_cell)

            except:
                # No action is necessary.
                pass

        # Read the next line
        s_line = o_gridded_file.readline()

    # Close the file and delete excess variables
    o_gridded_file.close()
    del o_gridded_file, s_line, sa_line_split, d_value_cell
    del dm_timestep_data_reshaped, da_gridded_data_temp

    # Promote the arrays to matrices
    dm_gridded_data = np.array(dm_gridded_data)
    da_gridded_timesteps = np.array(da_gridded_timesteps)

    return dm_gridded_data, da_gridded_timesteps

##### Assume the simulation completed successfully and attempt to parse the files ######################################
### Hydrograph ###
if b_hydrograph:
    ## Read and store the existing prn files ##
    l_previous_hydrographs = []

    for i_entry_file in range(0, len(sa_outputs_hydrograph), 1):
        # Open the output file
        o_hydrograph_file = open('ssf_' + sa_outputs_hydrograph[i_entry_file] + '_hydrograph.prn', 'r')

        # Loop and get the data from the file
        l_previous_temp = []
        for s_line in o_hydrograph_file:
            sa_line_split = s_line.split()
            l_previous_temp.append(sa_line_split)
        o_hydrograph_file.close()

        # Append the information into the previous hydrographs list
        l_previous_hydrographs.append(l_previous_temp)

    ## Change form numbers to indices ##
    ia_hydrograph_indices = np.array(ia_hydrograph_indices) - 1

    ## Read in the ohl file ##
    s_hydrograph_filename = os.path.join(s_project_name + '.ohl')

    try:
        # Attempt to read the ohl hydrograph file
        dm_hydrograph_full_data = np.loadtxt(s_hydrograph_filename, dtype=float)

        if len(dm_hydrograph_full_data) == 0:
            raise ImportException('Hydrograph file contains no data.')

    except:
        # Reading the ohl file has failed. Toggle the run status boolean
        b_outputReadFailure = True

    ## Continue only if the read was successful ##
    if not b_outputReadFailure:
        # Separate the data into the date and wanted columns
        da_hydrograph_timesteps = dm_hydrograph_full_data[:, 0]

        dm_hydrograph_data = np.zeros((len(da_hydrograph_timesteps), len(ia_hydrograph_indices)))
        for i_entry_column in range(0, len(ia_hydrograph_indices), 1):
            # +1 offset for the first date column
            dm_hydrograph_data[:, i_entry_column] = dm_hydrograph_full_data[:,
                                                    ia_hydrograph_indices[i_entry_column] + 1]

        # Convert the dates into mm/dd/yy hh:mm
        oa_hydrograph_timesteps = []
        for i_entry_timestep in range(0, len(da_hydrograph_timesteps), 1):
            # Get the year value
            i_year = np.floor(da_hydrograph_timesteps[i_entry_timestep])

            # Check if the year ia leap year
            b_leap_year = calendar.isleap(i_year)

            # Determine the number of seconds to add to the year
            i_number_of_seconds = np.NAN
            if b_leap_year:
                i_number_of_seconds = (366 * (da_hydrograph_timesteps[i_entry_timestep] - i_year) - 1) * 24 * 60 * 60
            else:
                i_number_of_seconds = (365 * (da_hydrograph_timesteps[i_entry_timestep] - i_year) - 1) * 24 * 60 * 60

            # Create a datetime object with just the year
            o_timestep = datetime.datetime(int(i_year), 1, 1)

            # Add the number of seconds to the year
            o_time_timestep = o_timestep + datetime.timedelta(seconds=i_number_of_seconds)

            # Append the timestamp into the data array
            oa_hydrograph_timesteps.append(o_time_timestep)

        del i_entry_timestep, i_year, b_leap_year, i_number_of_seconds, o_timestep, o_time_timestep

        ## Output the hydrographs to the prn ##
        for i_entry_file in range(0, len(sa_outputs_hydrograph), 1):
            # Open the output file
            o_hydrograph_output_file = open('ssf_' + sa_outputs_hydrograph[i_entry_file] + '_hydrograph.prn', 'w')

            for i_entry_timestep in range(0, len(oa_hydrograph_timesteps), 1):
                # Format the line to write
                s_line = sa_outputs_hydrograph[i_entry_file] + '   ' + \
                         oa_hydrograph_timesteps[i_entry_timestep].strftime('%m/%d/%Y   %H:%M:%S') + '   ' + \
                         str(dm_hydrograph_data[i_entry_timestep, i_entry_file]) + '\n'

                o_hydrograph_output_file.write(s_line)

            # Close the file
            o_hydrograph_output_file.close()

### WSE lake series ###
# Calculate the WSE time series
if b_wse_lake:
    # Check that the lengths are the same for the cell targets
    assert len(ia_wse_column_indices) == len(sa_outputs_wse)

    ## Read and store the existing prn files ##
    l_previous_wse = []

    for i_entry_file in range(0, len(ia_wse_column_indices), 1):
        # Open the output file
        o_wse_file = open('ssf_' + sa_outputs_wse[i_entry_file] + '_wse.prn', 'r')

        # Loop and get the data from the file
        l_previous_temp = []
        for s_line in o_wse_file:
            sa_line_split = s_line.split()
            l_previous_temp.append(sa_line_split)
        o_wse_file.close()

        # Append the information into the previous hydrographs list
        l_previous_wse.append(l_previous_temp)

    # Time series is active. Open the lel file
    ## Read in the lel file ##
    s_lel_filename = os.path.join(s_project_name + '.lel')

    try:
        # Read the elevation file
        dm_lel_inputs = np.loadtxt(s_lel_filename, dtype=float)

        if len(dm_lel_inputs) == 0:
            raise ImportException('WSE file contains no data.')

    except:
        # Elevation read failure. Switch to false
        b_outputReadFailure = True

    ## Continue only if the file read was successful ##
    if not b_outputReadFailure:
        # Output the WSE to the prn #
        for i_entry_file in range(0, len(ia_wse_column_indices), 1):
            # Open the output file
            o_wse_output_file = open('ssf_' + sa_outputs_wse[i_entry_file] + '_wse.prn', 'w')

            for i_entry_timestep in range(0, len(dm_lel_inputs), 1):
                # Format the time
                o_time_timestep = datetime.datetime(year=int(dm_lel_inputs[i_entry_timestep, 0]),
                                                    month=int(dm_lel_inputs[i_entry_timestep, 1]),
                                                    day=int(dm_lel_inputs[i_entry_timestep, 2]),
                                                    hour=int(dm_lel_inputs[i_entry_timestep, 3]),
                                                    minute=int(dm_lel_inputs[i_entry_timestep, 4]))

                # Format the line to write
                s_line = sa_outputs_wse[i_entry_file] + '   ' + o_time_timestep.strftime('%m/%d/%Y   %H:%M:%S') + \
                         '   ' + str(dm_lel_inputs[i_entry_timestep, ia_wse_column_indices[i_entry_file]]) + '\n'

                o_wse_output_file.write(s_line)

            # Close the file
            o_wse_output_file.close()


### Soil Moisture ###
# Calculate the soil moisture time series
if b_soil_moisture:
    # Check that the lengths are the same for the cell targets
    assert len(ia_soilm_cell_i) == len(ia_soilm_cell_j)

    ## Read and store the existing prn files ##
    l_previous_soilm = []

    for i_entry_file in range(0, len(ia_soilm_cell_i), 1):
        # Open the output file
        o_soilm_file = open('ssf_' + sa_outputs_soilm[i_entry_file] + '_soilm.prn', 'r')

        # Loop and get the data from the file
        l_previous_temp = []
        for s_line in o_soilm_file:
            sa_line_split = s_line.split()
            l_previous_temp.append(sa_line_split)
        o_soilm_file.close()

        # Append the information into the previous hydrographs list
        l_previous_soilm.append(l_previous_temp)

    # Time series is active. Extract the values for each of the target cells.
    # Change from GSSHA index to python index
    ia_soilm_cell_i = np.array(ia_soilm_cell_i) - 1
    ia_soilm_cell_j = np.array(ia_soilm_cell_j) - 1

    ## Read in the depth file as a function of time ##
    s_soilm_filename = os.path.join(s_project_name + '.sur')
    s_elevation_filename = os.path.join(s_project_name + '.ele')

    try:
        # Read in the soil moisture information from the sur file
        dm_soilm_data, da_soilm_timesteps = read_gridded_data(s_soilm_filename, s_elevation_filename)

    except:
        # Reading the soil moisture file has failed. Toggle the simulation status flag
        b_outputReadFailure = True

    ## Continue only if the read was successful ##
    if not b_outputReadFailure:
        # Read the time series for each of the cells #
        # Rows - Time, Columns - Cells
        dm_soilm_timeseries = np.zeros((len(da_soilm_timesteps), len(ia_soilm_cell_i)))

        for i_entry_cell in range(0, len(ia_soilm_cell_i), 1):
            for i_entry_timestep in range(0, len(da_soilm_timesteps), 1):
                dm_soilm_timeseries[i_entry_timestep, i_entry_cell] = \
                        dm_soilm_data[i_entry_timestep, ia_soilm_cell_i[i_entry_cell], ia_soilm_cell_j[i_entry_cell]]

        del i_entry_cell, i_entry_timestep

        # Output the soilm to the prn #
        for i_entry_file in range(0, len(ia_soilm_cell_i), 1):
            # Open the output file
            o_soilm_output_file = open('ssf_' + sa_outputs_soilm[i_entry_file] + '_soilm.prn', 'w')

            for i_entry_timestep in range(0, len(da_soilm_timesteps), 1):
                # Format the time
                o_time_timestep = o_time_start_gridded + datetime.timedelta(seconds=da_soilm_timesteps[i_entry_timestep])

                # Format the line to write
                s_line = sa_outputs_soilm[i_entry_file] + '   ' + o_time_timestep.strftime('%m/%d/%Y   %H:%M:%S') + \
                         '   ' + str(dm_soilm_timeseries[i_entry_timestep, i_entry_file]) + '\n'

                o_soilm_output_file.write(s_line)

            # Close the file
            o_soilm_output_file.close()


### Snow Depth ###
# Calculate the snow depth time series
if b_snow_depth:
    # Check that the lengths are the same for the cell targets
    assert len(ia_snowdepth_cell_i) == len(ia_snowdepth_cell_j)

    ## Read and store the existing prn files ##
    l_previous_snowdepth = []

    for i_entry_file in range(0, len(ia_snowdepth_cell_i), 1):
        # Open the output file
        o_snowdepth_file = open('ssf_' + sa_outputs_snowdepth[i_entry_file] + '_snowdepth.prn', 'r')

        # Loop and get the data from the file
        l_previous_temp = []
        for s_line in o_snowdepth_file:
            sa_line_split = s_line.split()
            l_previous_temp.append(sa_line_split)
        o_snowdepth_file.close()

        # Append the information into the previous hydrographs list
        l_previous_snowdepth.append(l_previous_temp)

    # Time series is active. Extract the values for each of the target cells.
    # Change from GSSHA index to python index
    ia_snowdepth_cell_i = np.array(ia_snowdepth_cell_i) - 1
    ia_snowdepth_cell_j = np.array(ia_snowdepth_cell_j) - 1

    ## Read in the depth file as a function of time ##
    s_snowdepth_filename = os.path.join(s_project_name + '.sdd')
    s_elevation_filename = os.path.join(s_project_name + '.ele')

    try:
        # Attempt to read the snow depth file
        dm_snowdepth_data, da_snowdepth_timesteps = read_gridded_data(s_snowdepth_filename, s_elevation_filename)

    except:
        # Error in reading the snow depth file. Toggle the simulation status flag
        b_outputReadFailure = True

    ## Continue only if the read was successful ##
    if not b_outputReadFailure:
        # Read the time series for each of the cells #
        # Rows - Time, Columns - Cells
        dm_snowdepth_timeseries = np.zeros((len(da_snowdepth_timesteps), len(ia_snowdepth_cell_i)))

        for i_entry_cell in range(0, len(ia_snowdepth_cell_i), 1):
            for i_entry_timestep in range(0, len(da_snowdepth_timesteps), 1):
                dm_snowdepth_timeseries[i_entry_timestep, i_entry_cell] = \
                        dm_snowdepth_data[i_entry_timestep, ia_snowdepth_cell_i[i_entry_cell], ia_snowdepth_cell_j[i_entry_cell]]

        del i_entry_cell, i_entry_timestep

        # Output the soilm to the prn #
        for i_entry_file in range(0, len(ia_snowdepth_cell_i), 1):
            # Open the output file
            o_snowdepth_output_file = open('ssf_' + sa_outputs_snowdepth[i_entry_file] + '_snowdepth.prn', 'w')

            for i_entry_timestep in range(0, len(da_snowdepth_timesteps), 1):
                # Format the time
                o_time_timestep = o_time_start_gridded + datetime.timedelta(seconds=da_snowdepth_timesteps[i_entry_timestep])

                # Format the line to write
                s_line = sa_outputs_snowdepth[i_entry_file] + '   ' + o_time_timestep.strftime('%m/%d/%Y   %H:%M:%S') + \
                         '   ' + str(dm_snowdepth_timeseries[i_entry_timestep, i_entry_file]) + '\n'

                o_snowdepth_output_file.write(s_line)

            # Close the file
            o_snowdepth_output_file.close()


### Soil Water Equivalent ###
# Calculate the snow depth time series
if b_snow_water_equivalent:
    # Check that the lengths are the same for the cell targets
    assert len(ia_swe_cell_i) == len(ia_swe_cell_j)

    ## Read and store the existing prn files ##
    l_previous_swe = []

    for i_entry_file in range(0, len(ia_swe_cell_i), 1):
        # Open the output file
        o_swe_file = open('ssf_' + sa_outputs_swe[i_entry_file] + '_swe.prn', 'r')

        # Loop and get the data from the file
        l_previous_temp = []
        for s_line in o_swe_file:
            sa_line_split = s_line.split()
            l_previous_temp.append(sa_line_split)
        o_swe_file.close()

        # Append the information into the previous hydrographs list
        l_previous_swe.append(l_previous_temp)

    # Time series is active. Extract the values for each of the target cells.
    # Change from GSSHA index to python index
    ia_swe_cell_i = np.array(ia_swe_cell_i) - 1
    ia_swe_cell_j = np.array(ia_swe_cell_j) - 1

    ## Read in the depth file as a function of time ##
    s_swe_filename = os.path.join(s_project_name + '.swd')
    s_elevation_filename = os.path.join(s_project_name + '.ele')

    try:
        # Attempt to read the file
        dm_swe_data, da_swe_timesteps = read_gridded_data(s_swe_filename, s_elevation_filename)

    except:
        # Error in reading the file. Toggle the simulation status boolean
        b_outputReadFailure = True

    ## Continue only if the read is successful ##
    if not b_outputReadFailure:
        # Read the time series for each of the cells #
        # Rows - Time, Columns - Cells
        dm_swe_timeseries = np.zeros((len(da_swe_timesteps), len(ia_swe_cell_i)))

        for i_entry_cell in range(0, len(ia_swe_cell_i), 1):
            for i_entry_timestep in range(0, len(da_swe_timesteps), 1):
                dm_swe_timeseries[i_entry_timestep, i_entry_cell] = \
                        dm_swe_data[i_entry_timestep, ia_swe_cell_i[i_entry_cell], ia_swe_cell_j[i_entry_cell]]

        del i_entry_cell, i_entry_timestep

        # Output the swe to the prn #
        for i_entry_file in range(0, len(ia_swe_cell_i), 1):
            # Open the output file
            o_swe_output_file = open('ssf_' + sa_outputs_swe[i_entry_file] + '_swe.prn', 'w')

            for i_entry_timestep in range(0, len(da_swe_timesteps), 1):
                # Format the time
                o_time_timestep = o_time_start_gridded + datetime.timedelta(seconds=da_swe_timesteps[i_entry_timestep])

                # Format the line to write
                s_line = sa_outputs_swe[i_entry_file] + '   ' + o_time_timestep.strftime('%m/%d/%Y   %H:%M:%S') + \
                         '   ' + str(dm_swe_timeseries[i_entry_timestep, i_entry_file]) + '\n'

                o_swe_output_file.write(s_line)

            # Close the file
            o_swe_output_file.close()



##### Handle the failed simulation status flag #########################################################################
if b_outputReadFailure:
    # Reading of at least one of the simulation result files has failed. This is assumed to be an invalid simulation
    # that should not be considered. Without action, the previous *.prn files will remain and the simulation will not
    # be shown to be invalid. This sets all values in the *.prn files to zero.

    if b_hydrograph:
        for i_entry_file in range(0, len(sa_outputs_hydrograph), 1):
            # Open the output file
            o_hydrograph_file = open('ssf_' + sa_outputs_hydrograph[i_entry_file] + '_hydrograph.prn', 'w')

            # Write each hydrograph file to disk with zero values
            for i_entry_list in range(0, len(l_previous_hydrographs[i_entry_file]), 1):
                for i_entry_row in range(0, len(l_previous_hydrographs[i_entry_file][i_entry_list]) - 1, 1):
                    o_hydrograph_file.write(l_previous_hydrographs[i_entry_file][i_entry_list][i_entry_row] + '   ')
                o_hydrograph_file.write('0.00000\n')

            o_hydrograph_file.close()

    if b_wse_lake:
        for i_entry_file in range(0, len(ia_wse_column_indices), 1):
            # Open the output file
            o_wse_file = open('ssf_' + sa_outputs_wse[i_entry_file] + '_wse.prn', 'w')

            # Write each WSE series to disk with zero values
            for i_entry_list in range(0, len(l_previous_wse[i_entry_file]), 1):
                for i_entry_row in range(0, len(l_previous_wse[i_entry_file][i_entry_list]) - 1, 1):
                    o_wse_file.write(l_previous_wse[i_entry_file][i_entry_list][i_entry_row] + '   ')
                o_wse_file.write('0.00000\n')
            o_wse_file.close()

    if b_soil_moisture:
        for i_entry_file in range(0, len(ia_soilm_cell_i), 1):
            # Open the output file
            o_soilm_file = open('ssf_' + sa_outputs_soilm[i_entry_file] + '_soilm.prn', 'w')

            # Write each soil moisture series to disk with zero values
            for i_entry_list in range(0, len(l_previous_soilm[i_entry_file]), 1):
                for i_entry_row in range(0, len(o_soilm_file[i_entry_file][i_entry_list]) - 1, 1):
                    o_soilm_file.write(l_previous_soilm[i_entry_file][i_entry_list][i_entry_row] + '   ')
                o_soilm_file.write('0.00000\n')
            o_soilm_file.close()

    if b_snow_depth:
        for i_entry_file in range(0, len(ia_snowdepth_cell_i), 1):
            # Open the output file
            o_snowdepth_file = open('ssf_' + sa_outputs_snowdepth[i_entry_file] + '_snowdepth.prn', 'w')

            # Write each snow depth series to disk with zero values
            for i_entry_list in range(0, len(l_previous_snowdepth[i_entry_file]), 1):
                for i_entry_row in range(0, len(o_snowdepth_file[i_entry_file][i_entry_list]) - 1, 1):
                    o_snowdepth_file.write(l_previous_snowdepth[i_entry_file][i_entry_list][i_entry_row] + '   ')
                o_snowdepth_file.write('0.00000\n')
            o_snowdepth_file.close()

    if b_snow_water_equivalent:
        # Replace each of the snow water equivalent files with a new file
        for i_entry_file in range(0, len(ia_swe_cell_i), 1):
            # Open the output file
            o_swe_file = open('ssf_' + sa_outputs_swe[i_entry_file] + '_swe.prn', 'w')

            # Write each snow depth series to disk with zero values
            for i_entry_list in range(0, len(l_previous_swe[i_entry_file]), 1):
                for i_entry_row in range(0, len(o_swe_file[i_entry_file][i_entry_list]) - 1, 1):
                    o_swe_file.write(l_previous_swe[i_entry_file][i_entry_list][i_entry_row] + '   ')
                o_swe_file.write('0.00000\n')
            o_swe_file.close()